import { EditorView } from '@codemirror/view';
import { HighlightStyle, syntaxHighlighting } from '@codemirror/language';
import { tags } from '@lezer/highlight';

const config = {
    name: 'nightOwl',
    dark: true,
    background: '#011627',
    foreground: '#d6deeb',
    selection: '#1d3b53',
    cursor: '#80a4c2',
    dropdownBackground: '#011627',
    dropdownBorder: '#5f7e97',
    activeLine: '#0003',
    matchingBracket: '#5f7e974d',
    keyword: '#c792ea',
    storage: '#c792ea',
    variable: '#d6deeb',
    parameter: '#ffcb8b',
    function: '#c5e478',
    string: '#ecc48d',
    constant: '#82AAFF',
    type: '#ffcb8b',
    class: '#ffcb8b',
    number: '#F78C6C',
    comment: '#637777',
    heading: '#82b1ff',
    invalid: '#EF5350',
    regexp: '#ecc48d',
};
const nightOwlTheme = EditorView.theme({
    '&': {
        color: config.foreground,
        backgroundColor: config.background,
    },
    '.cm-content': { caretColor: config.cursor },
    '&.cm-focused .cm-cursor': { borderLeftColor: config.cursor },
    '&.cm-focused .cm-selectionBackground, .cm-selectionBackground, & ::selection': { backgroundColor: config.selection },
    '.cm-panels': { backgroundColor: config.dropdownBackground, color: config.foreground },
    '.cm-panels.cm-panels-top': { borderBottom: '2px solid black' },
    '.cm-panels.cm-panels-bottom': { borderTop: '2px solid black' },
    '.cm-searchMatch': {
        backgroundColor: config.dropdownBackground,
        outline: `1px solid ${config.dropdownBorder}`
    },
    '.cm-searchMatch.cm-searchMatch-selected': {
        backgroundColor: config.selection
    },
    '.cm-activeLine': { backgroundColor: config.activeLine },
    '.cm-activeLineGutter': { backgroundColor: config.background },
    '.cm-selectionMatch': { backgroundColor: config.selection },
    '.cm-matchingBracket, .cm-nonmatchingBracket': {
        backgroundColor: config.matchingBracket,
        outline: 'none'
    },
    '.cm-gutters': {
        backgroundColor: config.background,
        color: config.foreground,
        border: 'none'
    },
    '.cm-lineNumbers, .cm-gutterElement': { color: 'inherit' },
    '.cm-foldPlaceholder': {
        backgroundColor: 'transparent',
        border: 'none',
        color: config.foreground
    },
    '.cm-tooltip': {
        border: `1px solid ${config.dropdownBorder}`,
        backgroundColor: config.dropdownBackground,
        color: config.foreground
    },
    '.cm-tooltip.cm-tooltip-autocomplete': {
        '& > ul > li[aria-selected]': {
            background: config.selection,
            color: config.foreground
        }
    },
}, { dark: config.dark });
const nightOwlHighlightStyle = HighlightStyle.define([
    { tag: tags.keyword, color: config.keyword },
    { tag: [tags.name, tags.deleted, tags.character, tags.macroName], color: config.variable },
    { tag: [tags.propertyName], color: config.function },
    { tag: [tags.processingInstruction, tags.string, tags.inserted, tags.special(tags.string)], color: config.string },
    { tag: [tags.function(tags.variableName), tags.labelName], color: config.function },
    { tag: [tags.color, tags.constant(tags.name), tags.standard(tags.name)], color: config.constant },
    { tag: [tags.definition(tags.name), tags.separator], color: config.variable },
    { tag: [tags.className], color: config.class },
    { tag: [tags.number, tags.changed, tags.annotation, tags.modifier, tags.self, tags.namespace], color: config.number },
    { tag: [tags.typeName], color: config.type, fontStyle: config.type },
    { tag: [tags.operator, tags.operatorKeyword], color: config.keyword },
    { tag: [tags.url, tags.escape, tags.regexp, tags.link], color: config.regexp },
    { tag: [tags.meta, tags.comment], color: config.comment },
    { tag: tags.strong, fontWeight: 'bold' },
    { tag: tags.emphasis, fontStyle: 'italic' },
    { tag: tags.link, textDecoration: 'underline' },
    { tag: tags.heading, fontWeight: 'bold', color: config.heading },
    { tag: [tags.atom, tags.bool, tags.special(tags.variableName)], color: config.variable },
    { tag: tags.invalid, color: config.invalid },
    { tag: tags.strikethrough, textDecoration: 'line-through' },
]);
const nightOwl = [
    nightOwlTheme,
    syntaxHighlighting(nightOwlHighlightStyle),
];

export { config, nightOwl, nightOwlHighlightStyle, nightOwlTheme };
